@@ -564,7 +564,7 @@ claim_op (GTlsConnectionBase    *tls,
           if (!do_implicit_handshake (tls, timeout, cancellable, error))
             {
               g_mutex_unlock (&priv->op_mutex);
-              g_tls_log_debug (tls, "claim_op failed: do_implicit_handshake failed");
+              g_tls_log_debug (tls, "claim_op failed: implicit handshake required");
               return FALSE;
             }
         }
@@ -886,6 +886,7 @@ g_tls_connection_base_base_check (GTlsConnectionBase *tls,
     return g_pollable_output_stream_is_writable (priv->base_ostream);
 
   g_assert_not_reached ();
+  return FALSE;
 }
 
 /* Checks whether the (D)TLS stream meets @condition; not the underlying base
@@ -1264,7 +1265,7 @@ verify_peer_certificate (GTlsConnectionBase *tls,
                                              NULL, &error);
       if (error)
         {
-          g_tls_log_warning (tls, "failure verifying certificate chain: %s", error->message);
+          g_tls_log_debug (tls, "failure verifying certificate chain: %s", error->message);
           g_assert (errors != 0);
           g_clear_error (&error);
         }
@@ -1318,38 +1319,52 @@ accept_or_reject_peer_certificate (gpointer user_data)
 
   update_peer_certificate_and_compute_errors (tls);
 
-  if (G_IS_TLS_CLIENT_CONNECTION (tls) && priv->peer_certificate)
+  if (priv->peer_certificate)
     {
-      GTlsCertificateFlags validation_flags;
+      if (G_IS_TLS_CLIENT_CONNECTION (tls))
+        {
+          GTlsCertificateFlags validation_flags;
 
-      if (!g_tls_connection_base_is_dtls (tls))
-        validation_flags =
-          g_tls_client_connection_get_validation_flags (G_TLS_CLIENT_CONNECTION (tls));
-      else
-        validation_flags =
-          g_dtls_client_connection_get_validation_flags (G_DTLS_CLIENT_CONNECTION (tls));
+          if (!g_tls_connection_base_is_dtls (tls))
+            validation_flags =
+              g_tls_client_connection_get_validation_flags (G_TLS_CLIENT_CONNECTION (tls));
+          else
+            validation_flags =
+              g_dtls_client_connection_get_validation_flags (G_DTLS_CLIENT_CONNECTION (tls));
 
-      if ((priv->peer_certificate_errors & validation_flags) == 0)
-        accepted = TRUE;
+          if ((priv->peer_certificate_errors & validation_flags) == 0)
+            accepted = TRUE;
+        }
+
+      if (!accepted)
+        {
+          gboolean sync_handshake_in_progress;
+
+          g_mutex_lock (&priv->op_mutex);
+          sync_handshake_in_progress = priv->sync_handshake_in_progress;
+          g_mutex_unlock (&priv->op_mutex);
+
+          if (sync_handshake_in_progress)
+            g_main_context_pop_thread_default (priv->handshake_context);
+
+          accepted = g_tls_connection_emit_accept_certificate (G_TLS_CONNECTION (tls),
+                                                               priv->peer_certificate,
+                                                               priv->peer_certificate_errors);
+
+          if (sync_handshake_in_progress)
+            g_main_context_push_thread_default (priv->handshake_context);
+        }
     }
-
-  if (!accepted)
+  else if (G_IS_TLS_SERVER_CONNECTION (tls))
     {
-      gboolean sync_handshake_in_progress;
+      GTlsAuthenticationMode mode = 0;
 
-      g_mutex_lock (&priv->op_mutex);
-      sync_handshake_in_progress = priv->sync_handshake_in_progress;
-      g_mutex_unlock (&priv->op_mutex);
+      g_object_get (tls,
+                    "authentication-mode", &mode,
+                    NULL);
 
-      if (sync_handshake_in_progress)
-        g_main_context_pop_thread_default (priv->handshake_context);
-
-      accepted = g_tls_connection_emit_accept_certificate (G_TLS_CONNECTION (tls),
-                                                           priv->peer_certificate,
-                                                           priv->peer_certificate_errors);
-
-      if (sync_handshake_in_progress)
-        g_main_context_push_thread_default (priv->handshake_context);
+      if (mode != G_TLS_AUTHENTICATION_REQUIRED)
+        accepted = TRUE;
     }
 
   priv->peer_certificate_accepted = accepted;
@@ -1363,9 +1378,6 @@ accept_or_reject_peer_certificate (gpointer user_data)
   g_cond_signal (&priv->verify_certificate_condition);
   g_mutex_unlock (&priv->verify_certificate_mutex);
 
-  g_object_notify (G_OBJECT (tls), "peer-certificate");
-  g_object_notify (G_OBJECT (tls), "peer-certificate-errors");
-
   return G_SOURCE_REMOVE;
 }
 
@@ -1521,10 +1533,16 @@ crank_sync_handshake_context (GTlsConnectionBase *tls,
   /* need_finish_handshake will be set inside sync_handshake_thread_completed(),
    * which should only ever be invoked while iterating the handshake context
    * here. So need_finish_handshake should only change on this thread.
+   *
+   * FIXME: This function is not cancellable. We should figure out how to
+   * support cancellation. We must not return from this function before it is
+   * safe to destroy handshake_context, but it's not safe to destroy
+   * handshake_context until after the handshake has completed. And the
+   * handshake operation is not cancellable, so we have a problem.
    */
   g_mutex_lock (&priv->op_mutex);
   priv->sync_handshake_in_progress = TRUE;
-  while (priv->sync_handshake_in_progress && !g_cancellable_is_cancelled (cancellable))
+  while (priv->sync_handshake_in_progress)
     {
       g_mutex_unlock (&priv->op_mutex);
       g_main_context_iteration (priv->handshake_context, TRUE);
