@@ -137,7 +137,14 @@ g_tls_connection_gnutls_initable_init (GInitable     *initable,
 
   ret = gnutls_certificate_allocate_credentials (&priv->creds);
   if (ret != GNUTLS_E_SUCCESS)
-    return FALSE;
+    {
+      g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_MISC,
+                   _("Could not create TLS connection: %s"),
+                   gnutls_strerror (ret));
+      g_clear_object (&base_io_stream);
+      g_clear_object (&base_socket);
+      return FALSE;
+    }
 
   gnutls_init (&priv->session, flags);
 
@@ -152,6 +159,8 @@ g_tls_connection_gnutls_initable_init (GInitable     *initable,
       g_set_error (error, G_TLS_ERROR, G_TLS_ERROR_MISC,
                    _("Could not create TLS connection: %s"),
                    gnutls_strerror (status));
+      g_clear_object (&base_io_stream);
+      g_clear_object (&base_socket);
       return FALSE;
     }
 
@@ -174,6 +183,9 @@ g_tls_connection_gnutls_initable_init (GInitable     *initable,
   if (flags & GNUTLS_DATAGRAM)
     gnutls_dtls_set_mtu (priv->session, 1400);
 
+  g_clear_object (&base_io_stream);
+  g_clear_object (&base_socket);
+
   return TRUE;
 }
 
