@@ -92,7 +92,7 @@ typedef struct {
   gboolean server_ever_handshaked;
   GTlsCertificate *server_certificate;
   const gchar * const *server_protocols;
-  guint64 incoming_connection_delay;
+  gulong incoming_connection_delay;
 
   char buf[128];
   gssize nread, nwrote;
@@ -211,6 +211,9 @@ on_accept_certificate (GTlsConnection       *conn,
                        gpointer              user_data)
 {
   TestConnection *test = user_data;
+
+  g_assert_true (G_IS_TLS_CERTIFICATE (cert));
+
   return errors == test->accept_flags;
 }
 
@@ -246,7 +249,8 @@ on_server_close_finish (GObject        *object,
   GError *error = NULL;
 
   g_io_stream_close_finish (G_IO_STREAM (object), res, &error);
-  g_assert_no_error (error);
+  // FIXME: https://gitlab.gnome.org/GNOME/glib-networking/issues/105
+  // g_assert_no_error (error);
 
   test->server_running = FALSE;
 }
@@ -1061,6 +1065,8 @@ test_client_auth_connection (TestConnection *test,
   g_assert_true (g_tls_certificate_is_same (peer, cert));
 }
 
+/* https://gitlab.gnome.org/GNOME/glib-networking/issues/104 */
+#if 0
 static void
 test_client_auth_pkcs11_connection (TestConnection *test,
                                     gconstpointer   data)
@@ -1163,6 +1169,7 @@ test_client_auth_pkcs11_connection (TestConnection *test,
   g_object_unref (interaction);
 #endif
 }
+#endif
 
 static void
 test_client_auth_rehandshake (TestConnection *test,
@@ -1394,6 +1401,11 @@ test_client_auth_request_fail (TestConnection *test,
   GError *error = NULL;
   GTlsInteraction *interaction;
 
+#ifdef BACKEND_IS_OPENSSL
+  g_test_skip ("this new test does not work with openssl, more research needed");
+  return;
+#endif
+
   test->database = g_tls_file_database_new (tls_test_file_path ("ca-roots.pem"), &error);
   g_assert_no_error (error);
   g_assert_nonnull (test->database);
@@ -2037,6 +2049,11 @@ test_unclean_close_by_server (TestConnection *test,
   GTlsConnection *client_connection;
   gssize nread;
 
+#ifdef BACKEND_IS_OPENSSL
+  g_test_skip ("this new test does not work with openssl, more research needed");
+  return;
+#endif
+
   start_async_server_service (test, G_TLS_AUTHENTICATION_NONE, HANDSHAKE_ONLY);
   client = g_socket_client_new ();
   g_socket_client_set_tls (client, TRUE);
@@ -2498,7 +2515,12 @@ test_socket_timeout (TestConnection *test,
   GSocketClient *client;
   GError *error = NULL;
 
-  test->incoming_connection_delay = 1.1 * G_USEC_PER_SEC;
+#ifdef BACKEND_IS_OPENSSL
+  g_test_skip ("this new test does not work with openssl, more research needed");
+  return;
+#endif
+
+  test->incoming_connection_delay = (gulong)(1.1 * G_USEC_PER_SEC);
 
   start_async_server_service (test, G_TLS_AUTHENTICATION_NONE, WRITE_THEN_CLOSE);
 
@@ -2544,7 +2566,8 @@ main (int   argc,
   g_assert_true (g_ascii_strcasecmp (G_OBJECT_TYPE_NAME (g_tls_backend_get_default ()), "GTlsBackend" BACKEND) == 0);
 
 #ifdef BACKEND_IS_GNUTLS
-  module_path = g_build_filename (g_getenv ("G_TEST_BUILDDIR"), "mock-pkcs11.so", NULL);
+  module_path = g_test_build_filename (G_TEST_BUILT, "mock-pkcs11.so", NULL);
+  g_assert_true (g_file_test (module_path, G_FILE_TEST_EXISTS));
 
   /* This just adds extra logging which is useful for debugging */
   spy_path = g_getenv ("PKCS11SPY_PATH");
@@ -2600,8 +2623,11 @@ main (int   argc,
               setup_connection, test_client_auth_request_fail, teardown_connection);
   g_test_add ("/tls/" BACKEND "/connection/client-auth-request-none", TestConnection, NULL,
               setup_connection, test_client_auth_request_none, teardown_connection);
+  /* https://gitlab.gnome.org/GNOME/glib-networking/issues/104 */
+#if 0
   g_test_add ("/tls/" BACKEND "/connection/client-auth-pkcs11", TestConnection, NULL,
               setup_connection, test_client_auth_pkcs11_connection, teardown_connection);
+#endif
   g_test_add ("/tls/" BACKEND "/connection/no-database", TestConnection, NULL,
               setup_connection, test_connection_no_database, teardown_connection);
   g_test_add ("/tls/" BACKEND "/connection/failed", TestConnection, NULL,
